(*** Simulation eines selbstorganisierenden Prozesses. c't 9/87 ***)

PROGRAM selforg;

  USES crt;

  CONST n      = 10;                         (* Anzahl der Zustandsvariablen *)
        igamma = 0.6;                              (* Anfangswert fuer Gamma *)
        decay  = 0.0006;                      (* Abklingkonstante fuer Gamma *)
        limit  = 0.0006;                       (* Abbruchschwelle fuer Gamma *)
        promax = 1200;                            (* Maximale Protokollaenge *)
        last   = true;
        bs     = $08;
        cr     = $0D;

  TYPE  pmat   = ARRAY [1..promax,0..n] OF integer;       
        svec   = ARRAY [1..n] OF integer;

  VAR   promat        : pmat;                             (* Protokollmatrix *)
        state         : svec;                              (* Zustandsvektor *)
        gamma         : real;                           (* Gewinnkoeffizient *)
        ranvar,                                           (* Zufallsvariable *)
        mindiff,diff,
        i,focus,trial : integer;
        distr         : char;
        outdat        : text;
        outdev        : STRING [15];

FUNCTION rannum : integer;
  VAR x : real;
BEGIN
  CASE distr OF
    '1': x := (random * 1000) + 50;
    '2': BEGIN
           x := 0;
           FOR I := 1 TO 4 DO
             x := x + (random * 1000) + 50;
           x := x / 4
         END
  END;
  rannum := round (x)
END;  (*** of rannum ***)

PROCEDURE prepare;

  VAR i : integer;

BEGIN
  writeln ('Simulation eines selbstorganisierenden Systems!');
  distr := '1';
  FOR i := 1 TO n DO
    state [i] := rannum;
  writeln;
  writeln ('Verteilung:   Gleich-   (1)');
  writeln ('              Normal-   (2)');
  writeln;
  write ('> <',chr (bs),chr(bs));
  REPEAT
    distr := readkey;
  UNTIL (distr IN ['1'..'2']);
  write (distr);
  writeln;
  writeln ('Durch-    Zufalls-     ');
  writeln ('-------------------------------- Zustandsvektor -------------------------------');
  write   ('Gang      Groesse      ');
  writeln ('1    2     3     4     5     6     7     8     9     10 ');
  write   ('====================');
  writeln ('===========================================================');
END;

PROCEDURE protocol (last : boolean);
  VAR i : integer;
BEGIN
  promat [trial,0] := round (ranvar);
  FOR i := 1 TO n DO
    promat [trial,i] := round (state [i]);
  IF (trial mod 10 = 0) OR last THEN
  BEGIN
    write (chr (cr),trial:4,'    ',ranvar:6,'    ');
    FOR i := 1 TO n DO
      write (state [i]:6);
    write (' ')
  END
END;

PROCEDURE cleanup;
  VAR i,j : integer;
BEGIN
  writeln;
  writeln;
  write ('Differenzen:          ');
  FOR i := 2 TO n DO
    write ((state [i] - state [i - 1]):6);
  writeln;
  writeln;
  write ('Wohin mit dem Protokoll? (Leere Eingabe = Kein Protokoll)   ');
  readln (outdev);
  IF (outdev <> '') THEN
  BEGIN
    writeln ('Anfertigung des Ablaufprotokolls...');
    assign (outdat,outdev);
    rewrite (outdat);
    FOR i := 1 TO trial DO
    BEGIN
      write (outdat,i:4,promat [i,0]:5,'     ');
      FOR j := 1 TO n DO
        write (outdat,promat [i,j]:5);
      writeln (outdat);
      IF (i mod 16 = 0) THEN
        write (char (13),i:5);
    END;
    writeln(char (13),i:5,' lines');
    close (outdat)
  END
END;



BEGIN (*** of MAIN ***)
  prepare;
  gamma := igamma;
  trial := 0;
  REPEAT
    trial := succ (trial);
    ranvar := rannum;
    mindiff := abs (ranvar - state [1]);
    focus := 1;
    FOR i := 2 TO n DO
    BEGIN
      diff := abs (ranvar - state [i]);
      IF (diff < mindiff) THEN
      BEGIN
        mindiff := diff;
        focus := i
      END
    END;
    FOR i := focus - 1 TO focus + 1 DO
      IF (i >= 1) AND (i <= n) THEN
        state [i] := state [i] + round (gamma * (ranvar - state [i]));
    protocol (NOT last);
    gamma := gamma - decay
  UNTIL (gamma <= limit) OR (trial = promax);
  protocol (last);
  cleanup
END.
