(*** Primitive Simulation des  'Waehlerverhaltens'  von  Peter Donnelly ***)
(*** (University College of Swansea)  und  Dominic Welsh (University of ***)
(*** Oxford) aus SPEKTRUM 6/85.  Ein zufaellig  ausgew{hlter  'Waehler' ***)
(*** nimmt die Meinung eines  zufaellig  ausgewaehlten  'Nachbarn'  an. ***)
(*** Diese  Prozedur  wird beliebig  oft wiederholt (-> 'time') und  in ***)
(*** einer Matrix laufend aktualisiert angezeigt. Daneben wird staendig ***)
(*** ueber den aktuellen 'Beliebtheitsgrad' beider Parteien informiert. ***)
(*** Claus Schoenleber, Kiel; claus@schoenleber.com                     ***)

PROGRAM election (input,output);

  USES crt;

  VAR hours,line,column,neighbor,
      n_line,n_column,time,minutes,
      stars,points                  : integer;
      num                           : integer;
      elector                       : ARRAY [1..15,1..15] OF char;
      party                         : string [1];


FUNCTION modulo16 (position : integer) : integer;

  VAR next_elector : integer;

BEGIN
  next_elector := position mod 16;
  IF position = 0 THEN
    next_elector := 15;
  IF position = 16 THEN
    next_elector := 1;
  modulo16 := next_elector
END;


PROCEDURE initialize;

  VAR x,y    : integer;

BEGIN
  stars := 0;
  points := 0;
  write ('How much hours : ');
  readln (hours);
  write ('initializing');
  FOR line := 1 TO 15 DO
  BEGIN
    write ('.');
    FOR column := 1 TO 15 DO
    BEGIN
      num := random (2) + 1;
      CASE num OF
        1 : BEGIN
              elector [column,line] := '*';
              stars := succ (stars)
            END;
        2 : BEGIN
              elector [column,line] := '.';
              points := succ (points)
            END
      END
    END
  END;
  clrscr;
  FOR y := 1 TO 15 DO
    FOR x := 1 TO 15 DO
    BEGIN
      gotoXY (2 * x + 20,y + 3);
      write (elector [x,y])
    END;
END;


PROCEDURE screen_output;

  VAR x,y : integer;

BEGIN
  gotoXY (1,1);
  write ('Time : ',time:4);
  gotoXY (1,3);
  writeln ('*-Party : ',stars:3);
  gotoXY (1,5);
  writeln ('.-Party : ',points:3);
  gotoXY (40,3);
  gotoXY (2 * column + 20,line + 3);
  write (elector [column,line])
END;


PROCEDURE runtime;

BEGIN
  FOR time := 0 TO hours DO
  BEGIN
    column   := random (15) + 1;
    line     := random (15) + 1;
    neighbor := random (8) + 1;
    n_column := column;
    n_line   := line;
    CASE neighbor OF
      1 : n_line := modulo16 (line - 1);
      2 : BEGIN
            n_column := modulo16 (column + 1);
            n_line := modulo16 (line - 1)
          END;
      3 : n_column := modulo16 (column + 1);
      4 : BEGIN
            n_column := modulo16 (column + 1);
            n_line := modulo16 (line + 1)
          END;
      5 : n_line := modulo16 (line + 1);
      6 : BEGIN
            n_column := modulo16 (column - 1);
            n_line := modulo16 (line + 1)
          END;
      7 : n_column := modulo16 (column - 1);
      8 : BEGIN
            n_column := modulo16 (column - 1);
            n_line := modulo16 (line - 1)
          END
    END;
    IF (elector [column,line] <> elector [n_column,n_line]) THEN
    BEGIN
      party := elector [n_column,n_line];
      CASE party [1] OF
        '*' : BEGIN
                stars := stars + 1;
                points := 225 - stars
              END;
        '.' : BEGIN
                points := points + 1;
                stars := 225 - points
              END
      END
    END;
    elector [column,line] := elector [n_column,n_line];
    screen_output
  END
END;

(*** $MAIN: election ***)

BEGIN
  clrscr;
  randomize;
  initialize;
  runtime
END.